##################################################################
##								##
##		   Graphiques et visualisation			##
##			de données avec R			##
##								##
##			      SCRIPT				##
##								##
##			Marina Doucerain			##
##		    marinadoucerain@gmail.com			##
##								##
##################################################################


library(car)
library(ggplot2)
library(corrplot)
library(lattice)
library(ggthemes)



### Charger le jeu de données ####

ss <- read.table("Rgraphs_jeu de données.txt")



### Exploration & diagnostiques ####

# Ne pas oublier de charger le module 'car'

# 1) Distributions ----

# Diagrammes à moustache
boxplot(salary.pre ~ visible, data=ss)
Boxplot(salary.pre ~ visible, data=ss, id.n=Inf, labels=as.character(ss$sex))

# Diagrammes quantile par quantile
qqnorm(ss$salary.pre)
with(ss, qqPlot(salary.pre, labels=sex, id.n=5, cex=.7))

# Histogramme
with(ss, {
	hist(salary.pre, freq=F, ylab="Densité", main=NULL, xlab="Salaire de départ")
	lines(density(salary.pre), lwd=2)
	rug(salary.pre)
})


# 2) Relations ----

# Nuages de points

plot(salary.pre ~ edu, data=ss)
scatterplot(salary.pre~edu, data=ss, id.n=5 )

scatterplot(salary.pre~edu | visible, data=ss, id.n=5, col=c("gray20", "blue") )

# Matrices de nuages de points

pairs(ss[,c("salary.pre", "salary.post", "edu", "yrs")])
scatterplotMatrix(~salary.pre+ salary.post+ edu+ yrs, data=ss, smooth=F, diagonal="density", ellipse=T, id.n=5) # Non montré lors de la présentation. Moins chargé que la matrice suivante.
scatterplotMatrix(~salary.pre+ salary.post+ edu+ yrs, data=ss, smooth=F, diagonal="density", ellipse=T, id.n=5, groups=ss$visible)

# Matrice de corrélation

# Ne pas oublier de charger le module 'corrplot'
corrplot(cor(mapply(as.numeric, ss[,c("yrs", "salary.pre", "edu", "salary.post", "visible", "sex")])), addCoef.col="white", order="AOE",
	 tl.col="black", tl.srt=45) 


# 3) Graphiques diagnostiques ----

## Modèle de régression ##

mod <- lm(salary.pre~visible+sex+edu+yrs*visible, data=ss)

# Résidus
residualPlots(mod)

# Diagrammes modèle marginal
marginalModelPlots(mod, cex=.5, pch=19)	

# Diagrammes variables ajoutées
avPlots(mod, id.n=2, id.cex=.8, id.col="blue", cex=.5)

# Influence
influenceIndexPlot(mod, cex=.5, id.n=10, id.col="red")

influencePlot(mod, id.n=2, id.col="red", id.cex=.7)


# 4) Identification

with(subset(ss, sex=="female" & visible=="yes"), {
	plot(salary.pre~yrs)
	identify(yrs, salary.pre, edu)
})

scatterplot(salary.pre~yrs, data=subset(ss, sex=="female" & visible=="yes"),
	    id.method="identify")



### lattice ####

# Fonction pour sélectionner des valeurs d'une variable allant de -2.5 écarts types à +2.5 écarts types par incréments de .25 écart type
vals <- function(x){a <- seq(from=-2.5, to=2.5, by=.25)
		    mean(x, na.rm=T)+a*sd(x, na.rm=T)
}

# Fonction pour choisir des valeurs (en écart types) d'une variable à fin de représentation graphique de l'effet de la variable.
subvals <- function(x, sourcevar, sds){
	a <- data.frame(b=mean(sourcevar, na.rm=T)+sds*sd(sourcevar, na.rm=T))
	for (i in a$b){
		a$c[a$b==i] <- x[which.min(abs(x-i))]
	}
	return(a$c)
}

# Création d'un jeu de données avec valeurs prédites #
pred <- expand.grid(visible=levels(ss$visible),
	    sex=levels(ss$sex), 
	    edu=vals(ss$edu),
	    yrs=vals(ss$yrs))
pred <- cbind(pred, predict(mod, newdata=pred, interval="confidence"))

head(pred)


# Effet des années vécues au Canada sur le salaire.
xyplot(fit +upr+lwr ~ yrs | sex, data=pred, type="l") # Problème à cause des variables edu et visible qui contiennent plusieurs valeurs dans le jeu de données 'pred'
xyplot(fit+upr+lwr ~ yrs | sex, data=subset(pred, edu==mean(ss$edu) & visible=="no"), type="l") # Spécifier un sous-ensemble avec seulement une valeur pour edu et visible résout le problème.

xyplot(fit ~ yrs | edu+sex, groups=visible, data=pred, type="l") # Le fonctionnement par défaut qui traite la variable continue edu par 'shingles' est cauchemardesque

xyplot(fit ~ yrs  | edu+ sex, groups=visible, data=subset(pred, edu %in% subvals(pred$edu, ss$edu, c(-1,1))), type="l", auto.key=T) # Spécifier un sous-ensemble avec seulement deux valeurs pour edu (+/-  1 écart type) résout le problème

# Ajouter des options
xyplot(fit ~ yrs  | factor(sex, labels=c("Femmes", "Hommes")) + factor(edu, labels=c("Scolarité: M -1ET", "Scolarité: M +1ET")), groups=visible, 
       data=subset(pred, edu %in% subvals(pred$edu, ss$edu, c(-1,1))), 
       type="l", 
       xlab=list("Nombre d'années au Canada", cex=1.5),
       ylab=list("Salaire", cex=1.5),
       layout=c(2,2),
       aspect="fill",
       between=list(x=.5, y=.5),
       auto.key=list(title="Minorité", cex.title=1.5, space="top", columns=2, points=F,
       	      lines=T, text=c("Non visible", "Visible"), cex=1.3),
       scales=list(alternating=1, tck=0),
       par.strip.text=list(cex=1.2, lines=1.3))


## Couleurs ##

show.settings()

# 1) Personnaliser localement

# Ne pas oublier de charger le module 'colorspace'
choose_palette()

# Ne pas oublier de charger le module 'RColorBrewer'
display.brewer.all()

mescouleurs <- brewer.pal(9,"Blues")

mesoptions <- list(
	superpose.line=list(col=mescouleurs[c(5,8)], lwd=2), 
	strip.background=list(col=mescouleurs[c(4,7)]),
	strip.border=list(col="black"))

xyplot(fit ~ yrs  | factor(sex, labels=c("Femmes", "Hommes")) + factor(edu, labels=c("Scolarité: M -1ET", "Scolarité: M +1ET")), groups=visible, 
       data=subset(pred, edu %in% subvals(pred$edu, ss$edu, c(-1,1))), 
       type="l", 
       xlab=list("Nombre d'années au Canada", cex=1.5),
       ylab=list("Salaire", cex=1.5),
       layout=c(2,2),
       aspect="fill",
       between=list(x=.5, y=.5),
       auto.key=list(title="Minorité", cex.title=1.5, space="top", columns=2, points=F,
       	      lines=T, text=c("Non visible", "Visible"), cex=1.3),
       scales=list(alternating=1, tck=0),
       par.strip.text=list(lines=1.1, col="white"),
       par.settings=mesoptions)


# 2) Personnaliser globalement: Créer son propre thème

trellis.par.get()

default <- trellis.par.get()

montheme <- trellis.par.get()
montheme$superpose.line$col <- brewer.pal(9,"Blues")[4:7]
montheme$superpose.line$lwd <- 2
montheme$superpose.polygon$col <- terrain_hcl(7)
montheme$strip.background$col <- brewer.pal(9, "Blues")[c(4,7,9)]
montheme$superpose.symbol$col <- brewer.pal(9, "Blues")[c(4,6,8)]
montheme$superpose.symbol$pch <- 19
montheme$strip.shingle$col <- heat_hcl(7)
montheme$plot.polygon$col <- "steelblue4"

trellis.par.set(montheme)

show.settings()



#### ggplot2 ####

ggplot(data=ss, aes(x=salary.pre)) + geom_histogram(colour="grey30")

# Données et mappage
p <- ggplot(data=ss, aes(x=posttest, fill=prog, colour=prog))

# Séparation représentation et transformation des données
p+geom_point(stat="bin")
p+geom_line(stat="bin")

# Défauts respectifs
p+geom_bar()
p+stat_density()

# Ajouter/modifier des mappages
p+geom_bar(aes(fill=sex, colour=NULL))

# Variables calculées par ggplot
p+geom_bar(aes(y=..density..))

# Paremètres supplémentaires
p+stat_density(position="identity", alpha=.5)

# Mappage vs. réglage
p+geom_area(stat="density", aes(fill=sex, colour=NULL))		# mappage
p+geom_area(stat="density", fill="grey", aes(colour=NULL))	# réglage (setting)

# Échelles: contrôlent mappage + guides visuels
p+stat_density(position="identity", alpha=.5, aes(colour=NULL)) + 
	scale_fill_manual(name="Programme", 
			  breaks=c("control", "immersion", "instruction"), 
			  labels=c("Contrôle", "Immersion", "Instruction"), 
			  values=mescouleurs[c(4,9,6)]) +
	scale_x_continuous(name="Note posttest",
			   breaks=c(25, 50, 75, 100),
			   labels=c("25%", "50%", "75%", "100%"))

# Couche après couche
q <- ggplot(subset(pred, sex=="female" & edu==mean(ss$edu) & yrs==mean(ss$yrs)),
	    aes(x=visible, y=fit))
q+ geom_jitter(data=subset(ss, sex=="female"), aes(x=visible, y=salary.pre), alpha=.5) +
	geom_point(colour="red", size=6) + geom_line(aes(group=1)) + 
	geom_errorbar(aes(ymin=lwr, ymax=upr), width=.2) + 
	scale_y_continuous(name="Salaire", limits=c(20000,44000)) + 
	scale_x_discrete(name="Minorité", breaks=c("no", "yes"), 
			 labels=c("Non visible", "Visible")) 

# Groupes: 'group'
r <- ggplot(data=subset(pred, sex=="male"& edu==mean(ss$edu)), aes(x=yrs, y=fit))
r + geom_point(data=ss, aes(y=salary.pre, colour=visible), size=1.5, alpha=.8) +
	geom_line(aes(group=visible, colour=visible), size=1) + 
	geom_ribbon(aes(ymin=lwr, ymax=upr, group=visible), alpha=.2) + 
	scale_colour_brewer(name="Minorité", breaks=c("no", "yes"), 
			    labels=c("Non-visible", "Visible"), palette="Set1") 

# Groupes: 'facet'
s <- ggplot(data=subset(pred, edu==mean(ss$edu)), aes(x=yrs, y=fit))
	
s + geom_point(data=ss, aes(y=salary.pre, colour=visible), size=1.5, alpha=.8) +
	geom_line(aes(group=visible, colour=visible), size=1) +
	geom_ribbon(aes(ymin=lwr, ymax=upr, group=visible), alpha=.2) +
	scale_colour_brewer(name="Minorité", breaks=c("no", "yes"), 
			    labels=c("Non-visible", "Visible"), palette="Set1") +
	facet_grid(.~sex)
	
# Annotations: 'annotate'
r+ geom_point(data=ss, aes(y=salary.pre, colour=visible), size=1.5, alpha=.8) +
	geom_line(aes(group=visible, colour=visible), size=1) + 
	geom_ribbon(aes(ymin=lwr, ymax=upr, group=visible), alpha=.2) + 
	scale_colour_brewer(name="Minorité", breaks=c("no", "yes"), 
			    labels=c("Non-visible", "Visible"), palette="Set1") + 
	annotate("text", x=10, y=20000, label="R^2 * '= .62'", size=6, parse=T)

# Thèmes: 'theme'
t <- r+ geom_point(data=ss, aes(y=salary.pre, colour=visible), size=1.5, alpha=.8) +
	geom_line(aes(group=visible, colour=visible), size=1) + 
	geom_ribbon(aes(ymin=lwr, ymax=upr, group=visible), alpha=.2) + 
	scale_colour_brewer(name="Minorité", breaks=c("no", "yes"), 
			    labels=c("Non-visible", "Visible"), palette="Set1") + 
	annotate("text", x=10, y=20000, label="R^2 * '= .62'", size=6, parse=T)

t+theme_bw()

t+theme_gray()

t+theme_wsj()	# Ne pas oublier de charger le module 'ggthemes'

default <- theme_set(theme_gray())

montheme <- theme_gray() + theme(panel.grid.major=element_blank(),
				 panel.grid.minor=element_blank(),
				 axis.text=element_text(colour="grey20", size=rel(1.5)),
				 axis.title=element_text(colour="grey20", size=rel(1.5)))

theme_set(montheme)
